package se.gawell.setrace.jersey;

/*
 *     This file is part of SeTrace.
 *
 *   SeTrace is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   SeTrace is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with SeTrace.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import java.net.URI;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import javax.ws.rs.ext.Providers;

import com.sun.jersey.api.client.AsyncViewResource;
import com.sun.jersey.api.client.AsyncWebResource;
import com.sun.jersey.api.client.ClientHandlerException;
import com.sun.jersey.api.client.ClientRequest;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.ViewResource;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.api.client.config.ClientConfig;
import com.sun.jersey.client.proxy.ViewProxy;
import com.sun.jersey.spi.MessageBodyWorkers;

/**
 * Expose com.sun.jersey.api.client.Client as an interface.
 * 
 * @author Niklas Gawell
 *
 */
public interface TracingJerseyBasedClient {

	/**
	 * Destroy the client. Any system resources associated with the client
	 * will be cleaned up.
	 * <p>
	 * This method must be called when there are not responses pending otherwise
	 * undefined behavior will occur.
	 * <p>
	 * The client must not be reused after this method is called otherwise
	 * undefined behavior will occur.
	 */
	public abstract void destroy();

	/**
	 * Get the {@link Providers} utilized by the client.
	 *
	 * @return the {@link Providers} utilized by the client.
	 */
	public abstract Providers getProviders();

	/**
	 * Get the {@link MessageBodyWorkers} utilized by the client.
	 *
	 * @return the {@link MessageBodyWorkers} utilized by the client.
	 */
	public abstract MessageBodyWorkers getMessageBodyWorkers();

	/**
	 * Create a Web resource from the client.
	 *
	 * @param u the URI of the resource.
	 * @return the Web resource.
	 */
	public abstract WebResource resource(String u);

	/**
	 * Create a Web resource from the client.
	 *
	 * @param u the URI of the resource.
	 * @return the Web resource.
	 */
	public abstract WebResource resource(URI u);

	/**
	 * Create an asynchronous Web resource from the client.
	 *
	 * @param u the URI of the resource.
	 * @return the Web resource.
	 */
	public abstract AsyncWebResource asyncResource(String u);

	/**
	 * Create an asynchronous Web resource from the client.
	 *
	 * @param u the URI of the resource.
	 * @return the Web resource.
	 */
	public abstract AsyncWebResource asyncResource(URI u);

	public abstract ViewResource viewResource(String u);

	public abstract ViewResource viewResource(URI u);

	public abstract AsyncViewResource asyncViewResource(String u);

	public abstract AsyncViewResource asyncViewResource(URI u);

	public abstract <T> T view(String u, Class<T> type);

	public abstract <T> T view(URI uri, Class<T> type);

	public abstract <T> T view(String u, T t);

	public abstract <T> T view(URI uri, T t);

	public abstract <T> Future<T> asyncView(String u, Class<T> type);

	public abstract <T> Future<T> asyncView(URI uri, Class<T> type);

	public abstract <T> Future<T> asyncView(String u, T t);

	public abstract <T> Future<T> asyncView(URI uri, T t);

	public abstract <T> T view(Class<T> c, ClientResponse response);

	public abstract <T> T view(T t, ClientResponse response);

	public abstract <T> ViewProxy<T> getViewProxy(Class<T> c);

	/**
	 * Set the {@link ExecutorService} for sending asynchronous
	 * HTTP requests when no underlying asynchronous HTTP implementation is
	 * utilized.
	 *
	 * @param es the {@link ExecutorService}.
	 * @since 1.4
	 */
	public abstract void setExecutorService(ExecutorService es);

	/**
	 * Get the {@link ExecutorService} for sending asynchronous
	 * HTTP requests when no underlying asynchronous HTTP implementation is
	 * utilized.
	 * <p>
	 * By default the implementation returned
	 * from {@link Executors#newCachedThreadPool() } is utilized.
	 *
	 * @return the {@link ExecutorService}.
	 * @since 1.4
	 */
	public abstract ExecutorService getExecutorService();

	/**
	 * Get the mutable property bag.
	 *
	 * @return the property bag.
	 */
	public abstract Map<String, Object> getProperties();

	/**
	 * Set if redirection should be performed or not.
	 *
	 * This method is the functional equivalent to setting the property
	 * {@link ClientConfig#PROPERTY_FOLLOW_REDIRECTS} on the property bag
	 * returned from {@link #getProperties}
	 *
	 * @param redirect if true then the client will automatically redirect
	 *        to the URI declared in 3xx responses.
	 */
	public abstract void setFollowRedirects(Boolean redirect);

	/**
	 * Set the read timeout interval, in milliseconds.
	 *
	 * This method is the functional equivalent to setting the property
	 * {@link ClientConfig#PROPERTY_READ_TIMEOUT} on the property bag
	 * returned from {@link #getProperties}
	 *
	 * @param interval the read timeout interval. If null or 0 then
	 * an interval of infinity is declared.
	 */
	public abstract void setReadTimeout(Integer interval);

	/**
	 * Set the connect timeout interval, in milliseconds.
	 *
	 * This method is the functional equivalent to setting the property
	 * {@link ClientConfig#PROPERTY_CONNECT_TIMEOUT} on the property bag
	 * returned from {@link #getProperties}
	 *
	 * @param interval the connect timeout interval. If null or 0 then
	 * an interval of infinity is declared.
	 */
	public abstract void setConnectTimeout(Integer interval);

	/**
	 * Set the client to send request entities using chunked encoding
	 * with a particular chunk size.
	 *
	 * This method is the functional equivalent to setting the property
	 * {@link ClientConfig#PROPERTY_CHUNKED_ENCODING_SIZE} on the property bag
	 * returned from {@link #getProperties}
	 *
	 * @param chunkSize the chunked encoding size. If &lt= 0 then the default
	 *        size will be used. If null then chunked encoding will not be
	 *        utilized.
	 */
	public abstract void setChunkedEncodingSize(Integer chunkSize);

	public abstract ClientResponse handle(final ClientRequest request)
			throws ClientHandlerException;

	/**
	 * Inject client-side bindings on an instance.
	 *
	 * @param o the instance to inject on.
	 */
	public abstract void inject(Object o);

}